import { __awaiter, __generator } from "tslib";
import { Canvas } from '@antv/g';
import { Renderer } from '@antv/g-webgl';
import { Plugin } from '@antv/g-plugin-gpgpu';
import { pageRank } from './link-analysis';
import { sssp } from './traversal';

var WebGPUGraph =
/** @class */
function () {
  function WebGPUGraph(options) {
    if (options === void 0) {
      options = {};
    }

    var canvas = options.canvas; // FIXME: use OffscreenCanvas instead of a real <canvas> DOM

    var $canvas = canvas || window.document.createElement('canvas'); // use WebGPU

    this.renderer = new Renderer({
      targets: ['webgpu']
    });
    this.renderer.registerPlugin(new Plugin()); // create a canvas

    this.canvas = new Canvas({
      canvas: $canvas,
      width: 1,
      height: 1,
      renderer: this.renderer
    });
  }

  WebGPUGraph.prototype.getDevice = function () {
    return __awaiter(this, void 0, void 0, function () {
      return __generator(this, function (_a) {
        switch (_a.label) {
          case 0:
            // wait for canvas' services ready
            return [4
            /*yield*/
            , this.canvas.ready];

          case 1:
            // wait for canvas' services ready
            _a.sent(); // get GPU Device


            return [2
            /*return*/
            , this.renderer.getDevice()];
        }
      });
    });
  };

  WebGPUGraph.prototype.pageRank = function (graphData, eps, alpha, maxIteration) {
    if (eps === void 0) {
      eps = 1e-5;
    }

    if (alpha === void 0) {
      alpha = 0.85;
    }

    if (maxIteration === void 0) {
      maxIteration = 1000;
    }

    return __awaiter(this, void 0, void 0, function () {
      var device;
      return __generator(this, function (_a) {
        switch (_a.label) {
          case 0:
            return [4
            /*yield*/
            , this.getDevice()];

          case 1:
            device = _a.sent();
            return [2
            /*return*/
            , pageRank(device, graphData, eps, alpha, maxIteration)];
        }
      });
    });
  };

  WebGPUGraph.prototype.sssp = function (graphData, sourceId, weightPropertyName) {
    if (weightPropertyName === void 0) {
      weightPropertyName = '';
    }

    return __awaiter(this, void 0, void 0, function () {
      var device;
      return __generator(this, function (_a) {
        switch (_a.label) {
          case 0:
            return [4
            /*yield*/
            , this.getDevice()];

          case 1:
            device = _a.sent();
            return [2
            /*return*/
            , sssp(device, graphData, sourceId, weightPropertyName)];
        }
      });
    });
  };

  WebGPUGraph.prototype.destroy = function () {
    if (this.canvas) {
      this.canvas.destroy();
    }
  };

  return WebGPUGraph;
}();

export { WebGPUGraph };